/**
 * 
 */
package br.com.datasind.util;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import org.jrimum.texgit.FlatFile;
import org.jrimum.texgit.Record;
import org.jrimum.utilix.text.Field;
import org.jrimum.utilix.text.Filler;

import br.com.datasind.entidade.EntidadeBancaria;
import br.com.datasind.entidade.GuiaAssistencial;
import br.com.datasind.entidade.Instituicao;
import br.com.datasind.entidade.Remessa;

/**
 * @author OsmarJunior 15:28:26
 */
public class UtilitarioRemessa {

	static DateFormat df = new SimpleDateFormat("ddMMyyyy");
	static DateFormat hf = new SimpleDateFormat("hhmmss");
	static Integer sequenciaLote = 1;
	static Integer sequenciaRegistroLote = 1;
	static Integer quantRegistros = 2;

	public static FlatFile<Record> geraRemessa(FlatFile<Record> ff, EntidadeBancaria entBancaria, Instituicao instituicao, Remessa remessa) throws Exception {

		ff.addRecord(criarCabecalho(ff, entBancaria, instituicao, remessa));
		ff.addRecord(criarCabecalhoLote(ff, entBancaria, remessa, instituicao));
		BigDecimal valorTotal = BigDecimal.ZERO;
		for (GuiaAssistencial guia : remessa.getGuias()) {
			ff.addRecord(adicionarSegmentoP(ff, guia, remessa, sequenciaLote, entBancaria));

			if (guia.getValorDocumento() != null) {
				valorTotal.add(guia.getValorDocumento());
			}
			sequenciaRegistroLote += 2;
			sequenciaLote++;
			quantRegistros += 2;
		}

		ff.addRecord(criarRadapeLote(ff, remessa, valorTotal));
		ff.addRecord(criarRadape(ff, remessa));

		return ff;
	}

	private static Record criarCabecalho(FlatFile<Record> ff, EntidadeBancaria entBancaria, Instituicao instituicao, Remessa remessa) throws Exception {

		Record linha = ff.createRecord("Header");
		// linha.setValue("IDReg", entBancaria.getBanco().getCodigo());
		linha.setValue("CPF_CNPJ", UtilitarioString.removeMascarasCeiCnpjCpf(instituicao.getCnpj()));
		linha.setValue("Agencia", entBancaria.getAgencia());
		linha.setValue("TipoInscricaoEmpresa", 2);
		if (entBancaria.getAgencia() == null || "".equals("")) {
			linha.setValue("DvAgencia", 0);
		} else {
			linha.setValue("DvAgencia", new Integer(entBancaria.getDvAgencia()));
		}

		linha.setValue("Convenio", entBancaria.getCedente());
		if (entBancaria.getNome().length() > 30) {
			linha.setValue("NomeCedente",
					entBancaria.getNome().toUpperCase().substring(0, 30));
		} else {
			linha.setValue("NomeCedente", entBancaria.getNome());
		}

		linha.setValue("SequenciaRemessa", remessa.getId());
		// linha.setValue("SequenciaRemessa", 1);
		if (remessa.getTesteDefinitiva()) {
			linha.setValue("ReservadoEmpresa", "REMESSA-TESTE");
		} else {
			linha.setValue("ReservadoEmpresa", "REMESSA-PRODUCAO");
		}

		linha.setValue("DataGravacao", df.format(remessa.getDataGeracao()));
		linha.setValue("HoraGravacao", hf.format(remessa.getDataGeracao()));

		return linha;
	}

	private static Record criarCabecalhoLote(FlatFile<Record> ff, EntidadeBancaria entBancaria, Remessa remessa, Instituicao instituicao) throws Exception {

		Record linha = ff.createRecord("HeaderLote");
		linha.setValue("IDReg", entBancaria.getBanco().getCodigo());

		linha.setValue("SequenciaLote", "0001");

		linha.setValue("TipoServico", "01");
		// linha.setValue("TipoServico", remessa.getTipoServico());
		linha.setValue("TipoInscricaoEmpresa", 2);
		linha.setValue("CPF_CNPJ", UtilitarioString.removeMascarasCeiCnpjCpf(instituicao.getCnpj()));
		linha.setValue("CodigoCedente", entBancaria.getCedente());
		linha.setValue("Agencia", entBancaria.getAgencia());

		if (entBancaria.getAgencia() == null || "".equals("")) {
			linha.setValue("DvAgencia", 0);
		} else {
			linha.setValue("DvAgencia", new Integer(entBancaria.getDvAgencia()));
		}
		linha.setValue("Convenio", entBancaria.getCedente());

		if (entBancaria.getNome().length() > 30) {
			linha.setValue("NomeCedente",
					entBancaria.getNome().toUpperCase().substring(0, 30));
		} else {
			linha.setValue("NomeCedente", entBancaria.getNome());
		}

		linha.setValue("DataGravacao", df.format(remessa.getDataGeracao()));
		linha.setValue("HoraGravacao", hf.format(remessa.getDataGeracao()));
		linha.setValue("SequenciaRemessa", remessa.getId());

		return linha;

	}

	private static Record adicionarSegmentoP(FlatFile<Record> ff, GuiaAssistencial guia, Remessa remessa, int sequencia, EntidadeBancaria entBancaria) throws Exception {

		// Boleto boleto = new Boleto();
		// Carteira carteira = new Carteira(14, TipoDeCobranca.COM_REGISTRO);

		Record linha = ff.createRecord("SegmentoP");
		linha.setValue("SequenciaLote", sequenciaLote);
		linha.setValue("SequenciaRegistroLote", sequenciaRegistroLote);
		linha.setValue("CodigoMovimento", guia.getCodigoMovimento());
		linha.setValue("CodigoMovimentacao", 01);
		linha.setValue("Agencia", entBancaria.getAgencia());
		linha.setValue("DvAgencia", new Integer(entBancaria.getDvAgencia()));

		linha.setValue("Convenio", entBancaria.getCedente());

		linha.setValue("NossoNumero", guia.getNossoNumero());

		linha.setValue("FormaCadastroTitulo", 1);

		// linha.setValue("FormaCadastroTitulo", TIPO_TITULO_SEM_REGISTRADO);

		if ((guia.getId() + "").length() > 5) {
			linha.setValue("SeuNumero", (guia.getId() + "").substring(0, 5));
		} else {
			linha.setValue("SeuNumero", (new Field<String>(guia.getId() + "", 5, Filler.ZERO_LEFT).write()));
		}

		if (entBancaria.getNome().length() > 30) {
			linha.setValue("NomeCedente",
					entBancaria.getNome().toUpperCase().substring(0, 30));
		} else {
			linha.setValue("NomeCedente", entBancaria.getNome());
		}

		linha.setValue("DataVencimento", df.format(guia.getDataVencimento()));

		if (guia.getValorDocumento() == null) {
			linha.setValue("ValorTitulo", BigDecimal.ZERO);
		} else {
			linha.setValue("ValorTitulo", UtilitarioString.removeMascarasCeiCnpjCpf(guia.getValorDocumento().setScale(2).toString()));
		}

		linha.setValue("DataEmissao", df.format(guia.getDataDocumento()));
		// Juros
		linha.setValue("DataMora", df.format(Utilitario.adicionarDiaNaData(guia.getDataVencimento(), 1)));
		linha.setValue("PeriodoCobMora", 2);
		linha.setValue("ValorMora", UtilitarioString.removeMascarasCeiCnpjCpf(guia.getTipoGuia().getJuros().toString()));

		// Desconto
		linha.setValue("TipoDesconto", "0");
		linha.setValue("DataDesconto", "00000000");

		// Código p/Protesto
		linha.setValue("Protesto", "3");
		linha.setValue("DiasProtesto", "90");

		// Código p/ Baixa/ Devolução

		linha.setValue("CodigoBaixa", "1");
		linha.setValue("DiasBaixa", "000");

		linha.addInnerRecord(adicionarSegmentoQ(ff, remessa, guia));

		return linha;

	}

	//
	private static Record adicionarSegmentoQ(FlatFile<Record> ff, Remessa remessa, GuiaAssistencial guia) throws Exception {

		// PessoaFisica pagador = dadosBoleto.getAvalista();
		//
		// if(pagador == null){
		// pagador = dadosBoleto.getSacado();
		// }

		Record linha = ff.createRecord("SegmentoQ");
		linha.setValue("SequenciaLote", sequenciaLote);
		linha.setValue("SequenciaRegistroLote", sequenciaRegistroLote + 1);
		linha.setValue("CodigoMovimento", guia.getCodigoMovimento());

		if (guia.getEmpresa().getId() != null) {
			linha.setValue("TipoInscricaoSacado", 2);
			linha.setValue("CPF_CNPJSacado", UtilitarioString.removeMascarasCeiCnpjCpf(guia.getEmpresa().getCnpj()));
			if (guia.getEmpresa().getRazaoSocial().length() > 40) {
				linha.setValue("NomeSacado", UtilitarioString.removeAcentos(guia.getEmpresa().getRazaoSocial().substring(0, 40)).toUpperCase());
			} else {
				linha.setValue("NomeSacado", UtilitarioString.removeAcentos(guia.getEmpresa().getRazaoSocial() + "").toUpperCase());
				// System.out.println(UtilitarioString.removeAcentos(guia.getEmpresa().getRazaoSocial()
				// + "").toUpperCase());
			}
			if (guia.getEmpresa().getEndereco().length() > 40) {
				linha.setValue("EnderecoSacado", UtilitarioString.removeAcentos(guia.getEmpresa().getEndereco().substring(0, 40)).toUpperCase());
			} else {
				linha.setValue("EnderecoSacado", UtilitarioString.removeAcentos(guia.getEmpresa().getEndereco() + "").toUpperCase());
			}

			if (guia.getEmpresa().getBairro().length() > 15) {
				linha.setValue("BairroSacado", UtilitarioString.removeAcentos(guia.getEmpresa().getBairro().substring(0, 15)).toUpperCase());
			} else {
				linha.setValue("BairroSacado", UtilitarioString.removeAcentos(guia.getEmpresa().getBairro() + "").toUpperCase());
			}

			String cep = guia.getEmpresa().getCep();
			if (cep == null || "".equals(cep)) {
				cep = guia.getInstituicao().getCep();
				if (cep == null || "".equals(cep)) {
					cep = "77240000";
				} else {
					cep = UtilitarioString.removeMascarasCeiCnpjCpf(cep);
				}
			} else {
				cep = UtilitarioString.removeMascarasCeiCnpjCpf(cep);
			}

			linha.setValue("CEPSacado", cep);

			if (guia.getEmpresa().getCidade().getDescCidade().length() > 15) {
				linha.setValue("CidadeSacado", UtilitarioString.removeAcentos(guia.getEmpresa().getCidade().getDescCidade().substring(0, 14)).toUpperCase());
			} else {
				linha.setValue("CidadeSacado", UtilitarioString.removeAcentos(guia.getEmpresa().getCidade().getDescCidade()).toUpperCase());
			}

			linha.setValue("UFSacado", UtilitarioString.removeAcentos(guia.getEmpresa().getCidade().getUf().getSiglaUF()).toUpperCase());

		} else {
			linha.setValue("TipoInscricaoSacado", 1);
			linha.setValue("CPF_CNPJSacado", UtilitarioString.removeMascarasCeiCnpjCpf(guia.getAssociado().getCPF()).toUpperCase());

			if (guia.getAssociado().getNome().length() > 40) {
				linha.setValue("NomeSacado", UtilitarioString.removeAcentos(guia.getAssociado().getNome().substring(0, 40)).toUpperCase());
			} else {
				linha.setValue("NomeSacado", UtilitarioString.removeAcentos(guia.getAssociado().getNome() + "").toUpperCase());
			}
			if (guia.getAssociado().getEndereco().length() > 40) {
				linha.setValue("EnderecoSacado", UtilitarioString.removeAcentos(guia.getAssociado().getEndereco().substring(0, 40)).toUpperCase());
			} else {
				linha.setValue("EnderecoSacado", UtilitarioString.removeAcentos(guia.getAssociado().getEndereco() + "").toUpperCase());
			}

			if (guia.getAssociado().getBairro().length() > 40) {
				linha.setValue("BairroSacado", UtilitarioString.removeAcentos(guia.getAssociado().getBairro().substring(0, 40)).toUpperCase());
			} else {
				linha.setValue("BairroSacado", UtilitarioString.removeAcentos(guia.getAssociado().getBairro() + "").toUpperCase());
			}

			String cep = guia.getEmpresa().getCep();
			if (cep == null || "".equals(cep)) {
				cep = guia.getInstituicao().getCep();
				if (cep == null || "".equals(cep)) {
					cep = "77240000";
				} else {
					cep = UtilitarioString.removeMascarasCeiCnpjCpf(cep);
				}
			} else {
				cep = UtilitarioString.removeMascarasCeiCnpjCpf(cep);
			}

			linha.setValue("CEPSacado", cep);
			if (guia.getAssociado().getCidade().getDescCidade().length() > 15) {
				linha.setValue("CidadeSacado", UtilitarioString.removeAcentos(guia.getAssociado().getCidade().getDescCidade().substring(0, 14)).toUpperCase());
			} else {
				linha.setValue("CidadeSacado", UtilitarioString.removeAcentos(guia.getAssociado().getCidade().getDescCidade()).toUpperCase());
			}

			linha.setValue("UFSacado", guia.getAssociado().getCidade().getUf().getSiglaUF());

			linha.setValue("TipoInscricaoAvalista", 0);

		}

		// Endereco endereco = pagador.getEnderecoPrincipal();

		// String enderecoString = "Endereço de Teste";
		// if (enderecoString.length() > 40)
		// enderecoString = enderecoString.substring(0, 39);
		// linha.setValue("EnderecoSacado", enderecoString);
		// linha.setValue("CEPSacado", "77064622");
		// linha.setValue("BairroSacado", "Bairro Teste");
		//
		// linha.setValue("CidadeSacado", "Palmas");
		//
		// linha.setValue("UFSacado", "TO");

		return linha;

	}

	private static Record criarRadapeLote(FlatFile<Record> ff, Remessa remessa, BigDecimal totalCobSimples) {

		// ++numeroSequencial;

		BigDecimal total = new BigDecimal(0);
		total = total.setScale(2, RoundingMode.HALF_UP);

		Record linha = ff.createRecord("TraillerLote");
		linha.setValue("SequenciaLote", "0001");
		linha.setValue("QtdLotes", (new Field<String>((quantRegistros) + "", 6, Filler.ZERO_LEFT).write()));
		linha.setValue("QtdRegistrosCobSimples", remessa.getGuias().size());
		linha.setValue("ValorTotalCobSimples", totalCobSimples);
		linha.setValue("QtdRegistrosCobCaucionada", 0);
		linha.setValue("ValorTotalCobCaucionada", 0);
		linha.setValue("QtdRegistrosCobDescontada", 0);
		linha.setValue("ValorTotalCobDescontada", 0);

		return linha;

	}

	private static Record criarRadape(FlatFile<Record> ff, Remessa remessa) {

		// ++numeroSequencial;

		Record linha = ff.createRecord("Trailler");
		linha.setValue("QtdLotes", remessa.getGuias().size());
		linha.setValue("QtdRegistros", (new Field<String>((quantRegistros + 2) + "", 6, Filler.ZERO_LEFT).write()));
		// linha.setValue("SequenciaRegistro", sequencia);
		return linha;

	}

}
